package ru.dbotthepony.mc.otm.triggers

import com.google.gson.JsonObject
import com.mojang.serialization.codecs.RecordCodecBuilder
import net.minecraft.advancements.Criterion
import net.minecraft.advancements.critereon.*
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.damagesource.DamageSource
import net.minecraft.world.entity.LivingEntity
import ru.dbotthepony.mc.otm.core.set
import ru.dbotthepony.mc.otm.data.Codec2TriggerSerializer
import ru.dbotthepony.mc.otm.data.DamagePredicateCodec
import java.util.Optional

abstract class HurtTrigger : SimpleCriterionTrigger<HurtTrigger.Instance>() {
	abstract val id: ResourceLocation

	final override fun createInstance(
		p_66248_: JsonObject,
		p_286603_: Optional<ContextAwarePredicate>,
		p_66250_: DeserializationContext
	): Instance {
		return codec.fromJson(p_66248_, p_286603_, p_66250_)
	}

	fun trigger(player: ServerPlayer, entity: LivingEntity, damage: Float, damageSource: DamageSource) {
		val context = EntityPredicate.createContext(player, entity)

		trigger(player) {
			it.predicate.map { it.matches(context) }.orElse(true) && it.damagePredicate.map { it.matches(player, damageSource, damage, damage, false) }.orElse(true)
		}
	}

	val codec = Codec2TriggerSerializer<Instance> { p ->
		RecordCodecBuilder.create {
			it.group(
				p.awareContextCodec.optionalFieldOf("predicate").forGetter(Instance::predicate),
				DamagePredicateCodec.optionalFieldOf("damagePredicate").forGetter(Instance::damagePredicate),
				p.playerPredicate.forGetter { it.playerPredicate() }
			).apply(it, ::Instance)
		}
	}

	inner class Instance(
		val predicate: Optional<ContextAwarePredicate> = Optional.empty(),
		val damagePredicate: Optional<DamagePredicate> = Optional.of(DamagePredicate(
			MinMaxBounds.Doubles.atLeast(1.0),
			MinMaxBounds.Doubles.atLeast(1.0),
			Optional.empty(),
			Optional.empty(),
			Optional.empty()
		)),
		player: Optional<ContextAwarePredicate> = Optional.empty()
	) : AbstractCriterionTriggerInstance(player) {
		override fun serializeToJson(): JsonObject {
			return codec.toJson(this)
		}

		fun criterion() = Criterion(this@HurtTrigger, this)
	}
}
